from datetime import datetime
from typing import List, Union

from pydantic import UUID4, BaseModel


class Image(BaseModel):
    uuid: UUID4

    class Config:
        orm_mode = True


class ItemCreatePrepareShipping(BaseModel):
    verification: bytes
    addressee: Union[str, None] = None
    team: Union[str, None] = None


class ItemCreateByImageAtStorage(BaseModel):
    image_uuid: UUID4
    storage_uuid: UUID4


class ItemCheckin(BaseModel):
    item_uuid: str
    storage_name: str
    amount: Union[int, None] = 1


class Item(BaseModel):
    uuid: UUID4
    amount: int
    created_at: datetime
    received_at: Union[datetime, None] = None

    addressee: Union[str, None] = None
    team: Union[str, None] = None
    images: List[Image] = []

    deployed: Union[str, None] = None
    deployed_at: Union[datetime, None] = None

    verification: Union[bytes, None] = None
    tag: Union[str, None] = None

    storage: Union[str, None] = None

    class Config:
        orm_mode = True


class Storage(BaseModel):
    name: str
    items: List[Item]

    class Config:
        orm_mode = True
